#include <sys/types.h>

/*	scan300c - Driver for Epson 300 C color scanner

Copyright (c) Terence P. Gliedt, 1992.
	This program may be freely redistributable in its original, unmodified form.
	The author grants you a nonexclusive, royalty-free license to use this software,
	and modify it as needed provided that this copyright notice remains, and any
	changes are clearly indicated as such.  No part may be incorporated in products or
	packages sold for profit without express permission of the copyright holder.
	The author assumes no responsibility for the consequences of using this program
	and no warranty of any sort is expressed or implied.
*/

/*	Global data areas. If MAIN is defined, we define the data, otherwise we reference them*/
#ifdef MAIN
char *MeName = "scan300c";
static char CopyRight[] = " Copyright (c) Terry Gliedt 1992";

int	Debug = 0;			/* Level of debugging that gets generated */
int	Quiet  = 0;			/* If set, we generate no wam fuzzy msgs */
int	Abort = 0;				/* If set, scan quits as soon as possible */
int	Error = 0;				/* Nonzero if any sort of error occurs */
int	LogicalUnit = 0;		/* To assign target,lun for sg0 */
int	Target = 0;			/* SCSI address. If this is zero, we try to guess */
int	Scanner;				/* File handle for scanner */
char	*ScannerDevice = DEVICE;		/* Default scanner device */
int	ScannerType;			/* Either SERIAL PARALLEL or SCSI */
#else
extern char *MeName;
extern int	Debug ;		
extern int	Quiet ;
extern int	Abort;
extern int	Error;
extern int LogicalUnit;
extern int Target;
extern int Scanner;
extern char *ScannerDevice;
extern int ScannerType;
#endif

/* Generate debugging information when -z specified */
#define DEBUG(s ,y1)	if (Debug) fprintf(stderr, s, y1);
#define DEBUG2(s ,y1,y2)	if (Debug) fprintf(stderr, s, y1, y2);
#define DEBUG3(s ,y1,y2, y3)	if (Debug) fprintf(stderr, s, y1, y2, y3);
#define DEBUG4(s ,y1,y2, y3, y4)	if (Debug) fprintf(stderr, s, y1, y2, y3, y4);
/* Extended debug.  Only effective if -z specified > 1 time */
#define XDEBUG(s ,y1)	if (Debug>1) fprintf(stderr, s, y1);
#define XDEBUG2(s ,y1,y2)	if (Debug>1) fprintf(stderr, s, y1, y2);
#define XDEBUG3(s ,y1,y2, y3)	if (Debug>1) fprintf(stderr, s, y1, y2, y3);
#define XDEBUG4(s ,y1,y2, y3, y4)	if (Debug>1) fprintf(stderr, s, y1, y2, y3, y4);

int DoSense();			/* Issue sense to scanner */
int DoRead();			/* Read from scanner */
int DoWrite();			/* Write to scanner */
int DoIoctl();			/* Do ioctl to device */
void DumpScsi_Req();	/* Dump ioctl blocks for debugging */
int FindScanner();		/* Search the SCSI bus for an Epson scanner */
char *Hex();			/* Format hexadecimal string */

/* Ways to be connected to scanner - ScannerType*/
#define	SERIAL		1
#define	PARALLEL	2
#define	SCSI		3

/*	Communication bytes */
#define	ESC		27
#define	ACK		6
#define	NAK		21
#define	STX		2
#define	CAN		24

/*	Commands to scanner */
#define	C_Request_Identity		'I'
#define	C_Request_Status		'F'
#define	C_Request_Condition	'S'
#define	C_Start_Scanning		'G'
#define	C_Set_Color_Mode		'C'
#define	C_Set_Data_Format	'D'
#define	C_Set_Resolution		'R'
#define	C_Set_Zoom			'H'
#define	C_Set_Reading_Area	'A'
#define	C_Set_Brightness		'L'
#define	C_Set_Gamma_Correction	'Z'
#define	C_Set_Halftoning_Mode		'B'
#define	C_Set_Color_Correction		'M'
#define	C_Initialize			'@'
#define	C_None				' '
#define	C_End				'e'		/* End of list in CmdList */

/* Information Data Block Header - returned by commands that return a data block */
struct  IDB {
	u_char	header;		/* STX or NAK */
	u_char	status;		/* Status byte */
	u_char	length[2];		/* Length of data to follow */
};

/* Use these masks ANDed with status in IDB */
#define	Status_Color	4+8
#define	Status_AreaEnd	32
#define	Status_Busy	64
#define	Status_Fatal	128

#define	Status_Color_Mono		0		/* Possible value of Status_Color */
#define	Status_Color_Green	1
#define	Status_Color_Red		2
#define	Status_Color_Blue		3

/* Data returned by an Identify */
struct  Identify_Data_Block {
	struct	IDB	header;
	char		identification[2];			/* B3 */
	char		res[57];					/* 19 sets of R + dpi */
	char		readingarea;				/* A */
	u_char	mainscanlength[2];			/* 3424 dots */
	u_char	subscanlength[2];			/* 4640 dots */
};

/* Data returned by an Request Status */
struct  Status_Data_Block {
	struct	IDB	header;
};

/* Data returned by an Request Condition */
struct  Condition_Data_Block {
	struct	IDB	header;
	char		C;			/* C - color mode */
	u_char	Color_Mode;
	char		R;			/* R */
	u_char 	outputmainscanresolution[2];
	u_char	outputsubscanresolution[2];
	char		A;			/* A */
	u_char	readingareamainscanoffset[2];
	u_char	 readingareasubscanoffset[2];
	u_char	readingareamainscanlength[2];
	u_char	readingareasubscanlength[2];
	char		D;			/* D */
	u_char	Format_Data;
	char		B;			/* B */
	u_char	HaltToningMode;
	char		L;			/* L */
	u_char	Brightness;
	char		Z;			/* Z */
	u_char	Gamma_Correction;
	char		H;			/* H */
	u_char	zoommainscan;
	u_char	zoomsubscan;
	char		M;			/* M */
	u_char	ColorCorrection;
};



